@@ -1,27 +1,42 @@
+from __future__ import annotations
+
+import socket
 from abc import abstractmethod
+from collections.abc import Callable, Collection, Mapping
+from contextlib import AsyncExitStack
 from io import IOBase
 from ipaddress import IPv4Address, IPv6Address
-from socket import AddressFamily, SocketType
-from typing import (
-    Any, AsyncContextManager, Callable, Collection, List, Optional, Tuple, TypeVar, Union)
+from socket import AddressFamily
+from types import TracebackType
+from typing import Any, Tuple, TypeVar, Union
 
-from .._core._typedattr import TypedAttributeProvider, TypedAttributeSet, typed_attribute
-from ._streams import ByteStream, Listener, T_Stream, UnreliableObjectStream
+from .._core._typedattr import (
+    TypedAttributeProvider,
+    TypedAttributeSet,
+    typed_attribute,
+)
+from ._streams import ByteStream, Listener, UnreliableObjectStream
 from ._tasks import TaskGroup
 
 IPAddressType = Union[str, IPv4Address, IPv6Address]
 IPSockAddrType = Tuple[str, int]
 SockAddrType = Union[IPSockAddrType, str]
 UDPPacketType = Tuple[bytes, IPSockAddrType]
-T_Retval = TypeVar('T_Retval')
+UNIXDatagramPacketType = Tuple[bytes, str]
+T_Retval = TypeVar("T_Retval")
 
 
 class _NullAsyncContextManager:
-    async def __aenter__(self):
+    async def __aenter__(self) -> None:
         pass
 
-    async def __aexit__(self, exc_type, exc_val, exc_tb):
-        pass
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> bool | None:
+        return None
 
 
 class SocketAttribute(TypedAttributeSet):
@@ -32,7 +47,7 @@ class SocketAttribute(TypedAttributeSet):
     #: for IP addresses, the local port the underlying socket is bound to
     local_port: int = typed_attribute()
     #: the underlying stdlib socket object
-    raw_socket: SocketType = typed_attribute()
+    raw_socket: socket.socket = typed_attribute()
     #: the remote address the underlying socket is connected to
     remote_address: SockAddrType = typed_attribute()
     #: for IP addresses, the remote port the underlying socket is connected to
@@ -41,16 +56,18 @@ class SocketAttribute(TypedAttributeSet):
 
 class _SocketProvider(TypedAttributeProvider):
     @property
-    def extra_attributes(self):
+    def extra_attributes(self) -> Mapping[Any, Callable[[], Any]]:
         from .._core._sockets import convert_ipv6_sockaddr as convert
 
-        attributes = {
+        attributes: dict[Any, Callable[[], Any]] = {
             SocketAttribute.family: lambda: self._raw_socket.family,
-            SocketAttribute.local_address: lambda: convert(self._raw_socket.getsockname()),
-            SocketAttribute.raw_socket: lambda: self._raw_socket
+            SocketAttribute.local_address: lambda: convert(
+                self._raw_socket.getsockname()
+            ),
+            SocketAttribute.raw_socket: lambda: self._raw_socket,
         }
         try:
-            peername = convert(self._raw_socket.getpeername())
+            peername: tuple[str, int] | None = convert(self._raw_socket.getpeername())
         except OSError:
             peername = None
 
@@ -60,15 +77,18 @@ class _SocketProvider(TypedAttributeProvider):
 
         # Provide local and remote ports for IP based sockets
         if self._raw_socket.family in (AddressFamily.AF_INET, AddressFamily.AF_INET6):
-            attributes[SocketAttribute.local_port] = lambda: self._raw_socket.getsockname()[1]
+            attributes[
+                SocketAttribute.local_port
+            ] = lambda: self._raw_socket.getsockname()[1]
             if peername is not None:
-                attributes[SocketAttribute.remote_port] = lambda: peername[1]
+                remote_port = peername[1]
+                attributes[SocketAttribute.remote_port] = lambda: remote_port
 
         return attributes
 
     @property
     @abstractmethod
-    def _raw_socket(self) -> SocketType:
+    def _raw_socket(self) -> socket.socket:
         pass
 
 
@@ -82,17 +102,17 @@ class SocketStream(ByteStream, _SocketProvider):
 
 class UNIXSocketStream(SocketStream):
     @abstractmethod
-    async def send_fds(self, message: bytes, fds: Collection[Union[int, IOBase]]) -> None:
+    async def send_fds(self, message: bytes, fds: Collection[int | IOBase]) -> None:
         """
         Send file descriptors along with a message to the peer.
 
         :param message: a non-empty bytestring
-        :param fds: a collection of files (either numeric file descriptors or open file or socket
-            objects)
+        :param fds: a collection of files (either numeric file descriptors or open file
+            or socket objects)
         """
 
     @abstractmethod
-    async def receive_fds(self, msglen: int, maxfds: int) -> Tuple[bytes, List[int]]:
+    async def receive_fds(self, msglen: int, maxfds: int) -> tuple[bytes, list[int]]:
         """
         Receive file descriptors along with a message from the peer.
 
@@ -113,18 +133,17 @@ class SocketListener(Listener[SocketStream], _SocketProvider):
     async def accept(self) -> SocketStream:
         """Accept an incoming connection."""
 
-    async def serve(self, handler: Callable[[T_Stream], Any],
-                    task_group: Optional[TaskGroup] = None) -> None:
+    async def serve(
+        self,
+        handler: Callable[[SocketStream], Any],
+        task_group: TaskGroup | None = None,
+    ) -> None:
         from .. import create_task_group
 
-        context_manager: AsyncContextManager
-        if task_group is None:
-            task_group = context_manager = create_task_group()
-        else:
-            # Can be replaced with AsyncExitStack once on py3.7+
-            context_manager = _NullAsyncContextManager()
+        async with AsyncExitStack() as stack:
+            if task_group is None:
+                task_group = await stack.enter_async_context(create_task_group())
 
-        async with context_manager:
             while True:
                 stream = await self.accept()
                 task_group.start_soon(handler, stream)
@@ -138,7 +157,10 @@ class UDPSocket(UnreliableObjectStream[UDPPacketType], _SocketProvider):
     """
 
     async def sendto(self, data: bytes, host: str, port: int) -> None:
-        """Alias for :meth:`~.UnreliableObjectSendStream.send` ((data, (host, port)))."""
+        """
+        Alias for :meth:`~.UnreliableObjectSendStream.send` ((data, (host, port))).
+
+        """
         return await self.send((data, (host, port)))
 
 
@@ -148,3 +170,25 @@ class ConnectedUDPSocket(UnreliableObjectStream[bytes], _SocketProvider):
 
     Supports all relevant extra attributes from :class:`~SocketAttribute`.
     """
+
+
+class UNIXDatagramSocket(
+    UnreliableObjectStream[UNIXDatagramPacketType], _SocketProvider
+):
+    """
+    Represents an unconnected Unix datagram socket.
+
+    Supports all relevant extra attributes from :class:`~SocketAttribute`.
+    """
+
+    async def sendto(self, data: bytes, path: str) -> None:
+        """Alias for :meth:`~.UnreliableObjectSendStream.send` ((data, path))."""
+        return await self.send((data, path))
+
+
+class ConnectedUNIXDatagramSocket(UnreliableObjectStream[bytes], _SocketProvider):
+    """
+    Represents a connected Unix datagram socket.
+
+    Supports all relevant extra attributes from :class:`~SocketAttribute`.
+    """
