@@ -1,23 +1,30 @@
+from __future__ import annotations
+
 import pytest
+from _pytest.logging import LogCaptureFixture
 from _pytest.pytester import Pytester
 
 from anyio import get_all_backends
 
 pytestmark = pytest.mark.filterwarnings(
-    "ignore:The TerminalReporter.writer attribute is deprecated:pytest.PytestDeprecationWarning:"
+    "ignore:The TerminalReporter.writer attribute is deprecated"
+    ":pytest.PytestDeprecationWarning:"
 )
 
-pytest_args = "-v", "-p", "anyio", "-p", "no:asyncio"
+pytest_args = "-v", "-p", "anyio", "-p", "no:asyncio", "-p", "no:trio"
 
 
 def test_plugin(testdir: Pytester) -> None:
     testdir.makeconftest(
         """
+        from contextvars import ContextVar
         import sniffio
         import pytest
 
         from anyio import sleep
 
+        var = ContextVar("var")
+
 
         @pytest.fixture
         async def async_fixture():
@@ -25,6 +32,13 @@ def test_plugin(testdir: Pytester) -> None:
             return sniffio.current_async_library()
 
 
+        @pytest.fixture
+        async def context_variable():
+            token = var.set("testvalue")
+            yield var
+            var.reset(token)
+
+
         @pytest.fixture
         async def some_feature():
             yield None
@@ -59,20 +73,26 @@ def test_plugin(testdir: Pytester) -> None:
         async def test_skip_inline(some_feature):
             # Test for github #214
             pytest.skip("Test that skipping works")
+
+        @pytest.mark.anyio
+        async def test_contextvar(context_variable):
+            # Test that a contextvar set in an async fixture is visible to the test
+            assert context_variable.get() == "testvalue"
         """
     )
 
     result = testdir.runpytest(*pytest_args)
     result.assert_outcomes(
-        passed=3 * len(get_all_backends()), skipped=len(get_all_backends())
+        passed=4 * len(get_all_backends()), skipped=len(get_all_backends())
     )
 
 
-def test_asyncio(testdir: Pytester) -> None:
+def test_asyncio(testdir: Pytester, caplog: LogCaptureFixture) -> None:
     testdir.makeconftest(
         """
         import asyncio
         import pytest
+        import threading
 
 
         @pytest.fixture(scope='class')
@@ -96,6 +116,16 @@ def test_asyncio(testdir: Pytester) -> None:
             yield None
             asyncio.get_running_loop().call_soon(callback)
             await asyncio.sleep(0)
+
+        @pytest.fixture
+        def no_thread_leaks_fixture():
+            # this has to be non-async fixture so that it wraps up
+            # after the event loop gets closed
+            threads_before = threading.enumerate()
+            yield
+            threads_after = threading.enumerate()
+            leaked_threads = set(threads_after) - set(threads_before)
+            assert not leaked_threads
         """
     )
 
@@ -114,7 +144,11 @@ def test_asyncio(testdir: Pytester) -> None:
                 await asyncio.sleep(0)
                 return anyio_backend
 
-            def test_class_fixture_in_test_method(self, async_class_fixture, anyio_backend_name):
+            def test_class_fixture_in_test_method(
+                self,
+                async_class_fixture,
+                anyio_backend_name
+            ):
                 assert anyio_backend_name == 'asyncio'
                 assert async_class_fixture == 'asyncio'
 
@@ -134,11 +168,23 @@ def test_asyncio(testdir: Pytester) -> None:
 
         async def test_callback_exception_during_teardown(teardown_fail_fixture):
             pass
+
+        async def test_exception_handler_no_exception():
+            asyncio.get_event_loop().call_exception_handler(
+                {"message": "bogus error"}
+            )
+            await asyncio.sleep(0.1)
+
+        async def test_shutdown_default_executor(no_thread_leaks_fixture):
+            # Test for github #503
+            asyncio.get_event_loop().run_in_executor(None, lambda: 1)
         """
     )
 
     result = testdir.runpytest(*pytest_args)
-    result.assert_outcomes(passed=2, failed=1, errors=2)
+    result.assert_outcomes(passed=4, failed=1, errors=2)
+    assert len(caplog.messages) == 1
+    assert caplog.messages[0] == "bogus error"
 
 
 def test_autouse_async_fixture(testdir: Pytester) -> None:
