@@ -1,11 +1,12 @@
+from __future__ import annotations
+
 import asyncio
-import sys
 import threading
 import time
 from concurrent.futures import Future
 from contextvars import ContextVar
 from functools import partial
-from typing import Any, List, NoReturn, Optional
+from typing import Any, NoReturn
 
 import pytest
 import sniffio
@@ -21,11 +22,6 @@ from anyio import (
     wait_all_tasks_blocked,
 )
 
-if sys.version_info < (3, 7):
-    current_task = asyncio.Task.current_task
-else:
-    current_task = asyncio.current_task
-
 pytestmark = pytest.mark.anyio
 
 
@@ -60,30 +56,31 @@ async def test_run_in_thread_exception() -> None:
 
 
 async def test_run_in_custom_limiter() -> None:
-    num_active_threads = max_active_threads = 0
+    max_active_threads = 0
 
     def thread_worker() -> None:
-        nonlocal num_active_threads, max_active_threads
-        num_active_threads += 1
-        max_active_threads = max(num_active_threads, max_active_threads)
+        nonlocal max_active_threads
+        active_threads.add(threading.current_thread())
+        max_active_threads = max(max_active_threads, len(active_threads))
         event.wait(1)
-        num_active_threads -= 1
+        active_threads.remove(threading.current_thread())
 
     async def task_worker() -> None:
         await to_thread.run_sync(thread_worker, limiter=limiter)
 
     event = threading.Event()
     limiter = CapacityLimiter(3)
+    active_threads: set[threading.Thread] = set()
     async with create_task_group() as tg:
         for _ in range(4):
             tg.start_soon(task_worker)
 
         await sleep(0.1)
-        assert num_active_threads == 3
+        assert len(active_threads) == 3
         assert limiter.borrowed_tokens == 3
         event.set()
 
-    assert num_active_threads == 0
+    assert len(active_threads) == 0
     assert max_active_threads == 3
 
 
@@ -96,11 +93,11 @@ async def test_cancel_worker_thread(
     cancellable: bool, expected_last_active: str
 ) -> None:
     """
-    Test that when a task running a worker thread is cancelled, the cancellation is not acted on
-    until the thread finishes.
+    Test that when a task running a worker thread is cancelled, the cancellation is not
+    acted on until the thread finishes.
 
     """
-    last_active: Optional[str] = None
+    last_active: str | None = None
 
     def thread_worker() -> None:
         nonlocal last_active
@@ -156,11 +153,11 @@ async def test_asynclib_detection() -> None:
 
 @pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_asyncio_cancel_native_task() -> None:
-    task: "Optional[asyncio.Task[None]]" = None
+    task: asyncio.Task[None] | None = None
 
     async def run_in_thread() -> None:
         nonlocal task
-        task = current_task()
+        task = asyncio.current_task()
         await to_thread.run_sync(time.sleep, 0.2, cancellable=True)
 
     async with create_task_group() as tg:
@@ -174,9 +171,9 @@ def test_asyncio_no_root_task(asyncio_event_loop: asyncio.AbstractEventLoop) ->
     """
     Regression test for #264.
 
-    Ensures that to_thread.run_sync() does not raise an error when there is no root task, but
-    instead tries to find the top most parent task by traversing the cancel scope tree, or failing
-    that, uses the current task to set up a shutdown callback.
+    Ensures that to_thread.run_sync() does not raise an error when there is no root
+    task, but instead tries to find the top most parent task by traversing the cancel
+    scope tree, or failing that, uses the current task to set up a shutdown callback.
 
     """
 
@@ -226,7 +223,7 @@ def test_asyncio_run_sync_no_asyncio_run(
     def exception_handler(loop: object, context: Any = None) -> None:
         exceptions.append(context["exception"])
 
-    exceptions: List[BaseException] = []
+    exceptions: list[BaseException] = []
     asyncio_event_loop.set_exception_handler(exception_handler)
     asyncio_event_loop.run_until_complete(to_thread.run_sync(time.sleep, 0))
     assert not exceptions
@@ -260,8 +257,8 @@ def test_asyncio_no_recycle_stopping_worker(
         await event1.wait()
         asyncio_event_loop.call_soon(event2.set)
         await anyio.to_thread.run_sync(time.sleep, 0)
-        # At this point, the worker would be stopped but still in the idle workers pool, so the
-        # following would hang prior to the fix
+        # At this point, the worker would be stopped but still in the idle workers pool,
+        # so the following would hang prior to the fix
         await anyio.to_thread.run_sync(time.sleep, 0)
 
     event1 = asyncio.Event()
@@ -269,3 +266,17 @@ def test_asyncio_no_recycle_stopping_worker(
     task1 = asyncio_event_loop.create_task(taskfunc1())
     task2 = asyncio_event_loop.create_task(taskfunc2())
     asyncio_event_loop.run_until_complete(asyncio.gather(task1, task2))
+
+
+async def test_stopiteration() -> None:
+    """
+    Test that raising StopIteration in a worker thread raises a RuntimeError on the
+    caller.
+
+    """
+
+    def raise_stopiteration() -> NoReturn:
+        raise StopIteration
+
+    with pytest.raises(RuntimeError, match="coroutine raised StopIteration"):
+        await to_thread.run_sync(raise_stopiteration)
